local AddonName, GridIndicatorsDynamic = ...
local Grid = Grid
local GridFrame = Grid:GetModule("GridFrame")
local Media = LibStub("LibSharedMedia-3.0")
local db = Grid.db:GetNamespace("GridIndicatorsDynamic")

local strsub = string.utf8sub or string.sub

local function Text_NewIndicator(frame)
	local text = frame:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
	return text
end

local function Text_ResetIndicator(self)
	local profile = db.profile[self.__id]
	if not profile then
		return
	end
	local font = Media:Fetch("font", profile.font) or STANDARD_TEXT_FONT

	local frame = self.__owner
	local bar = frame.indicators.bar

	self:SetParent(bar)
	self:SetFont(font, profile.fontSize - 3, profile.fontOutline)

	if profile.fontShadow then
		self:SetShadowOffset(1, -1)
	else
		self:SetShadowOffset(0, 0)
	end

	if profile.invertBarColor and profile.invertTextColor then
		self:SetShadowColor(1, 1, 1)
	else
		self:SetShadowColor(0, 0, 0)
	end
	local anchor = profile.anchor
	self:ClearAllPoints()
	if string.match(anchor, "LEFT") then 
		self:SetJustifyH("LEFT")
	elseif string.match(anchor, "RIGHT") then
		self:SetJustifyH("RIGHT")
	else
		self:SetJustifyH("CENTER")
	end
	
	self:SetJustifyV("CENTER")
	self:SetPoint(anchor, profile.offsetX, profile.offsetY)
end

local function Text_SetStatus(self, color, text, value, maxValue, texture, texCoords, stack, start, duration)
	local profile = db.profile[self.__id]

	if not text or text == "" then
		return self:SetText("")
	end

	self:SetText(strsub(text, 1, profile.textlength))

	if color then
		if profile.invertBarColor and profile.invertTextColor then
			self:SetTextColor(color.r * 0.2, color.g * 0.2, color.b * 0.2, color.a or 1)
		else
			self:SetTextColor(color.r, color.g, color.b, color.a or 1)
		end
	end
end

local function Text_ClearStatus(self)
	self:SetText("")
end

function GridIndicatorsDynamic:Text_RegisterIndicator(id, name)
	GridFrame:RegisterIndicator(id, name,
		Text_NewIndicator,
		Text_ResetIndicator,
		Text_SetStatus,
		Text_ClearStatus
	)
end